local AddOn, NS = ...

------------------------------------------------------- LOCAL FUNCTIONS ---------------------------------------------
local POWERTABLE = {
	[Enum.PowerType.Mana] = "manacolor", --0
	[Enum.PowerType.Rage] = "ragecolor", --1
	[Enum.PowerType.Focus] = "focuscolor", --2
	[Enum.PowerType.Energy] = "energycolor", --3
	--[SPELL_POWER_COMBO_POINTS] = --4
	--[SPELL_POWER_RUNES] = 5
	[Enum.PowerType.RunicPower] = "runicpowercolor", --6
	--[SPELL_POWER_SOUL_SHARDS] = 7
	[Enum.PowerType.LunarPower] = "holycolor", --8
	[Enum.PowerType.HolyPower] = "lunarpowercolor", --9
	--[SPELL_POWER_ALTERNATE_POWER] = 10
	[Enum.PowerType.Maelstrom] = "maelstrompower", --11
	--[SPELL_POWER_CHI] = 12
	[Enum.PowerType.Insanity] = "insanitycolor", --13
	--[SPELL_POWER_OBSOLETE] = 14
	--[SPELL_POWER_OBSOLETE2] = 15
	--[SPELL_POWER_ARCANE_CHARGES] = 16
	[Enum.PowerType.Fury] = "furycolor" --17
	--[SPELL_POWER_PAIN] = 18
}

local PetHappy = {
	[1] = DUF_TEXT.Unhappy,
	[3] = DUF_TEXT.Content,
	[3] = DUF_TEXT.Happy,
}

local mathceil, mathfloor = math.ceil, math.floor
local strfind, strformat, strgsub, strlen, strsub = string.find, string.format, string.gsub, string.len, string.sub

local function DUF_UnitSex(unit)
	value = UnitSex(unit)
	if (value == 2) then
		value = DUF_TEXT.Male
	elseif (value == 3) then
		value = DUF_TEXT.Female
	else
		value = DUF_TEXT.Neuter
	end
	return value
end

local function DUF_gsub(text, variable, value)
	if (value) then
		text = strgsub(text, variable, value)
	elseif (strfind(text, " "..variable)) then
		text = strgsub(text, " "..variable, "")
	else
		text = strgsub(text, variable, "")
	end
	return text
end

local function Delete_If_Not(text, variable, stop)
	local varstart, stopstart = 0, strlen(text)
	while varstart do
		varstart = strfind(text, variable)
		if varstart then
			local _, en = strfind(text, "$EN", varstart)
			if stop then
				local stopend = strfind(text, stop, varstart)
				if stopend then
					stopstart = stopend - 1
				end
			end
			if en and en < stopstart then
				stopstart = en
			end
			text = DUF_gsub(text, strsub(text, varstart, stopstart), "")
		end
	end
	text = DUF_gsub(text, "$EN", "")
	return text
end

local function FormatValue(value, format, short)
	local val = tonumber(value)
	local formatAt
	local compound = not short -- temporary if we want to use the compound xm xxx.xk or just x.xm
	if format == 3 then -- Comma formatted non decimal
--		return BreakUpLargeNumbers(val)
		return FormatLargeNumber(val)
	elseif format == 1 then
		formatAt = 10000
	elseif format == 2 then
		formatAt = 1000
	else
		formatAt = 100000
	end
	if val < formatAt then
--		return BreakUpLargeNumbers(val)
		return FormatLargeNumber(val)
	elseif val < 1000000 then -- Less than 1 meg
		return DL_round(value/1000, 1) .. "k"
	elseif compound and val >= 1000000 and val < 100000000 then -- Less than 100m and is compound ( xm xxx.xk )
		local hiVal = mathfloor(val / 1000000)
		local lowVal = val - (hiVal * 1000000)
		if lowVal < formatAt then
--			return tostring(hiVal) .. "m " .. BreakUpLargeNumbers(lowVal)
			return tostring(hiVal) .. "m " .. FormatLargeNumber(lowVal)
		else
			return tostring(hiVal) .. "m " .. DL_round(lowVal/1000, 1) .. "k"
		end
	else
		return DL_round(val/1000000, 2) .. "m"
	end
end

local function DUF_Format_TimeLeft(timeLeft)
	local timeText
	if (timeLeft > 60) then
		local seconds = timeLeft - math.floor(timeLeft / 60) * 60
		if (seconds < 10) then
			timeText = math.floor(timeLeft / 60)..":0"..seconds
		else
			timeText = math.floor(timeLeft / 60)..":"..seconds
		end
	else
--		if (timeLeft < 10) then
--			timeText = "0" .. timeLeft .. "s"
--		else
			timeText = timeLeft .. "s"
--		end
	end
	return timeText
end

--[[
local CLASSES = {
	MALE = {},
	FEMALE = {},
}
FillLocalizedClassList(CLASSES.MALE, false) 
FillLocalizedClassList(CLASSES.FEMALE, true)
local SEX = { "MALE", "MALE", "FEMALE", }
]]--
------------------------------------------------------- END LOCAL FUNCTIONS -----------------------------------------

NS.DUF_VARIABLE_FUNCTIONS = {
-- Deprecated Variables (return "") - Removed Variables are at the end.
	-- Pet Happiness
	["$ph"] = { func = function(text, unit)
			local value = NS.GetPetHappiness()
			value = value and PetHappy[value] or ""
			text = DUF_gsub(text, '$ph', value)
			return text
		end,
		events = {}},

	-- Pet Current XP
	["$px"] = { func = function(text, unit, self)
			local value = NS.GetPetExperience()
			value = value or ""
			text = DUF_gsub(text, '$px', value)
			return text
		end,
		events = {}},

	-- Pet XP Needed to Level
	["$py"] = { func = function(text, unit, self)
			local _, value = NS.GetPetExperience()
			value = value or ""
			text = DUF_gsub(text, '$py', value)
			return text
		end,
		events = {} 
		},

	-- Pet XP Percent Complete
	["$pc"] = { func = function(text, unit, self)
			local min, max = NS.GetPetExperience()
			local value = ""
			if (max and min and max > 0) then
				value = DL_round(min / max * 100, 0)
			end
			text = DUF_gsub(text, '$pc', value)
			return text
		end,
		events = {}},

	-- Pet XP Percent Needed
		["$pp"] = { func = function(text, unit, self) -- Not in Localization.lua
			local min, max = NS.GetPetExperience()
			local value = ""
			if (max and min and max > 0) then
				value = DL_round(min / max * 100, 0)
				value = 100 - value
			end
			text = DUF_gsub(text, '$pp', value)
			return text
		end,
		events = {}},

	-- Pet XP To Go
		["$pg"] = { func = function(text, unit, self) -- Not in Localization.lua
			local min, max = NS.GetPetExperience()
			local value = ""
			if (max and min and max > 0) then
				value = max - min
			end
			text = DUF_gsub(text, '$pg', value)
			return text
		end,
		events = {}},

-- Current Variables
	-- IF Player
	["$PL"] = { func = function(text, unit, self)
			local player = UnitIsPlayer(unit)
			if not player then
				text = Delete_If_Not(text, "$PL", "$NP")
				return text
			end
			text = DUF_gsub(text, '$PL', "")
			return text
		end,
		events = {} },
	-- IF NPC
	["$NP"] = { func = function(text, unit, self)
			local npc = not UnitIsPlayer(unit)
			if not npc then
				text = Delete_If_Not(text, "$NP", "$PL")
				return text
			end
			text = DUF_gsub(text, '$NP', "")
			return text
		end, 
		events = {} },
	-- Unit Role
	["$ur"] = { func = function(text, unit, self)
				local role = NS.UnitGroupRolesAssigned(unit)
				if (role == "TANK") then
					role = DUF_TEXT.Tank
				elseif (role == "HEALER") then
					role = DUF_TEXT.Healer
				elseif (role == "DAMAGER") then
					role = DUF_TEXT.Damage
				else
					role = ""
				end
				text = DUF_gsub(text, '$ur', role)
				return text
			end,
			events = {"PLAYER_FLAGS_CHANGED"} },
	-- Real Mana - Druid
	["$rm"] = { func = function(text, unit, self)
				local altmana = UnitPower(unit, Enum.PowerType.Mana)
				local txt = FormatValue(altmana, self.formatM, self.shortMana)
				text = DUF_gsub(text, '$rm', txt)
				return text
			end,
			events = {"UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },			
			
	-- Real Mana Max - Druid
	["$rx"] = { func = function(text, unit, self)
				local altmanamax = UnitPowerMax(unit, Enum.PowerType.Mana)
				local txt = FormatValue(altmanamax, self.formatM, self.shortMana)
				text = DUF_gsub(text, '$rx', txt)
				return text
			end,
			events = {"UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },			

	-- Real Mana Percent - Druid
	["$rp"] = { func = function(text, unit, self)
				local altmanapercent
				if (UnitPowerMax(unit, Enum.PowerType.Mana) == 0) then
					altmanapercent = mathfloor(UnitPower(unit, Enum.PowerType.Mana)*100)
				else
					altmanapercent = mathfloor(UnitPower(unit, Enum.PowerType.Mana) / UnitPowerMax(unit, Enum.PowerType.Mana)*100)
				end
				text = DUF_gsub(text, '$rp', altmanapercent)
				return text
			end,
			events = {"UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },			

	-- Title of Player 
	["$tp"] = { func = function(text, unit, self)
			local value =  GetTitleName(GetCurrentTitle())
			text = DUF_gsub(text, '$tp', value)
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },

	-- PVP Countdown Timer
	["$pf"] = { func = function(text, unit, self)
				if (not unit == "player") then return end
				local pTimer
				if (not UnitIsPVP(unit)) then
					pTimer = ""
					self.pvpcd = nil
				else
					self.pvpcd = 1
					pTimer = mathfloor(GetPVPTimer()/1000)
					if (pTimer == 301) then -- Permanently flagged
						pTimer = DUF_TEXT.PVPForever
					else
						pTimer = DUF_Format_TimeLeft(pTimer - 1)
					end
				end
				text = DUF_gsub(text, '$pf', pTimer)
				return text
			end,
			events = {"UNIT_FACTION"} },

	-- Realm Name
	["$rn"] = { func = function(text, unit, self)
				local punit, prealm = UnitName(unit)
				if (not prealm) then
					prealm = GetRealmName()
				end
				text = DUF_gsub(text, '$rn', prealm)
				return text
			end,
			events = {"UNIT_NAME_UPDATE"} },
	-- New Line
	["$nl"] = { func = function(text, unit, self)
				text = strgsub(text, '$nl', "\n")
				return text
			end,
			events = {} },
	-- Raid Group
	["$rg"] = { func = function(text, unit, self)
				local value
				for i=1, GetNumGroupMembers() do
					if (UnitIsUnit("raid"..i, unit)) then
						_, _, value = GetRaidRosterInfo(i)
						break
					end
				end
				text = DUF_gsub(text, '$rg', value)
				return text
			end,
			events = {"RAID_ROSTER_UPDATE"} },
	-- Honor Progress
	["$hn"] = { func = function(text, unit, self)
				local value = NS.GetPVPRankProgress()
				value = math.floor(value * 100)
				text = string.gsub(text, '$hn', value)
				return text
			end,
			events = {"PLAYER_PVP_KILLS_CHANGED", "PLAYER_PVP_RANK_CHANGED"} },
	-- Name
	["$nm"] = { func = function(text, unit, self)
				local unitname = UnitName(unit)
				if (not unitname) then unitname = unit end
				self.TextString:SetText(nil)
				text = DUF_gsub(text, '$nm', unitname)
				return text
			end,
			events = {"UNIT_NAME_UPDATE"} },

	-- Name including title
	["$nr"] = { func = function(text, unit, self)
				local unitname = UnitPVPName(unit)
				if (not unitname) then unitname = unit end
				text = DUF_gsub(text, '$nr', unitname)
				return text
			end,
			events = {"UNIT_NAME_UPDATE"} },

	-- Level
	["$lv"] = { func = function(text, unit, self)
				local level = UnitLevel(unit)
				if (level == -1) then
					if (UnitClassification(unit) == "worldboss") then
						level = GetMaxPlayerLevel()+11 --"71+"
					else
						level = ">"..(UnitLevel("player") + 10)
					end
				end
				text = DUF_gsub(text, '$lv', level)
				return text
			end,
		events = {"UNIT_LEVEL"} },

	-- Next Level
	["$xn"] = { func = function(text, unit, self)
				local level = UnitLevel(unit) + 1
				text = DUF_gsub(text, '$xn', level)
				return text
			end,
		events = {"UNIT_LEVEL"} },	

	-- Player Class
	["$cl"] = { func = function(text, unit, self)
			local class = UnitClass(unit)
			if (class == "") then class = "class" end
			if (UnitIsPlayer(unit)) then
				text = DUF_gsub(text, '$cl', class)
			else
				text = DUF_gsub(text, '$cl', "")
			end
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },

	-- Mob Class
	["$cy"] = { func = function(text, unit, self)
--			local _, class = UnitClass(unit)
			local class = UnitClass(unit)
--			local sex = UnitSex(unit)
			if (not class) then 
				class = "class" 
--			else
--				class = CLASSES[SEX[sex]][class]
			end
			if (not UnitIsPlayer(unit)) then
				text = DUF_gsub(text, '$cy', class)
			else
				text = DUF_gsub(text, '$cy', "")
			end
			return text
		end,
		events = {} },

	-- Current Health
	["$hc"] = { func = function(text, unit, self)
			local health = NS:DUF_Get_Health(unit)
			text = DUF_gsub(text, '$hc', health)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT} },

	-- Current Health in thousands
	["$hk"] = { func = function(text, unit, self)
			local health = NS:DUF_Get_Health(unit)
			local txt = FormatValue(health, self.formatH, self.shortHealth)
			text = DUF_gsub(text, '$hk', txt)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT} },

	-- Dynamic Health 
	["$hd"] = { func = function(text, unit, self) 
		local currValue,maxValue = UnitHealth(unit),UnitHealthMax(unit) 
--		local debug = 0
		if maxValue == 0 then 
		   return DUF_gsub(text, '$hd', "") 
		end 
		local perc = currValue/maxValue * 100 
		local MHfound = false 
		if ( UnitIsDead(unit) ) then 
			return DUF_gsub("Dead", '$hd', "") 
		elseif ( UnitIsGhost(unit) ) then 
			return DUF_gsub("Ghost", '$hd', "") 
		elseif ( not UnitIsConnected(unit) or maxValue == 1 ) then 
			return DUF_gsub("Disc", '$hd', "") 
		end 
		if ( not UnitIsFriend("player", unit) ) then 
		end 
		if currValue > maxValue then 
			maxValue = currValue 
		end 
		local hpDiff = maxValue - currValue 
		if not flag and MHfound and not UnitIsFriend("player", unit) then 
			return DUF_gsub(DUF_FormatLargeValue(currValue) .." (".. perc .."%)", '$hd', value) 
		end 
		if not (flag == 1) and (MHfound or unit == "pet" or unit == "player" or UnitInParty(unit) or UnitInRaid(unit)) then 
			if flag == 2 then 
				if hpDiff > 0 then 
					return DUF_gsub(DUF_FormatLargeValue(currValue).."$coffff7f7f-".. DUF_FormatLargeValue(hpDiff) .."|r", '$hd', "") 
				else 
					return DUF_gsub(DUF_FormatLargeValue(currValue), '$hd', value) 
				end 
			elseif flag == 3 and hpDiff > 0 then 
				return DUF_gsub("$coffff7f7f-"..DUF_FormatLargeValue(hpDiff).."|r", '$hd', "") 
			elseif flag == 3 then 
				return "" 
			elseif flag == 4 then 
				return DUF_gsub(DUF_FormatLargeValue(currValue), '$hd', value) 
			else 
				if (maxValue-currValue)>0 then 
					text = DUF_gsub(text, '$hd', DUF_FormatLargeValue(maxValue).." |cFFFF8888-"..DUF_FormatLargeValue(maxValue-currValue)) 
				else 
					text = DUF_gsub(text, '$hd', DUF_FormatLargeValue(maxValue)) 
				end 
				return text 
			end 
		else 
			return strformat("%.0f%%", perc) 
		end 
		text = DUF_gsub(text, '$hd', value) 
		return text 
	end, 
	events = {NS.UNIT_HEALTH_EVENT} },

	-- Max Health
	["$hm"] = { func = function(text, unit, self)
			local healthmax = NS:DUF_Get_MaxHealth(unit)
			text = DUF_gsub(text, '$hm', healthmax)
			return text
		end,
		events = {"UNIT_MAXHEALTH"} },

	-- Max Health in thousands
	["$kh"] = { func = function(text, unit, self)
			local healthmax = NS:DUF_Get_MaxHealth(unit)
			local txt = FormatValue(healthmax, self.formatH, self.shortHealth)
			text = DUF_gsub(text, '$kh', txt)
			return text
		end,
		events = {"UNIT_MAXHEALTH"} },

	-- Damage Taken, i.e. lost health
	["$dt"] = { func = function(text, unit, self)
			local damage = NS:DUF_Get_HealthDamage(unit)
			text = DUF_gsub(text, '$dt', damage)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT, "UNIT_MAXHEALTH"} },

	-- Damage as a percent
	["$dp"] = { func = function(text, unit, self)
			local damage = UnitHealthMax(unit) - UnitHealth(unit)
			local healthmax = UnitHealthMax(unit)
			if (healthmax == 0) then
				damage = 0
			else
				damage = mathfloor(damage/healthmax * 100)
			end
			text = DUF_gsub(text, '$dp', damage)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT, "UNIT_MAXHEALTH"} },

	-- Health as a percent
	["$hp"] = { func = function(text, unit, self)
			local percent = 0
			local health = UnitHealth(unit)
			local healthmax = UnitHealthMax(unit)
			if (healthmax > 0) then
				percent = mathfloor(health/healthmax * 100)
			else
				percent = 0
			end
			text = DUF_gsub(text, '$hp', percent)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT, "UNIT_MAXHEALTH"} },

	-- Current Mana
	["$mc"] = { func = function(text, unit, self)
			local mana = UnitPower(unit)
			text = DUF_gsub(text, '$mc', mana, 1)
			return text
		end,
		events = {"UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },

	-- Current Mana in thousands
	["$mk"] = { func = function(text, unit, self)
			local mana = UnitPower(unit)
			local txt = FormatValue(mana, self.formatM, self.shortMana)
			text = DUF_gsub(text, '$mk', txt, 1)
			return text
		end,
		events = {"UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },

	-- Max Mana
	["$mm"] = { func = function(text, unit, self)
			local manamax = UnitPowerMax(unit)
			if (not manamax) then
				manamax = 0
			end
			text = DUF_gsub(text, '$mm', manamax, 1)
			return text
		end,
		events = {"UNIT_MAXPOWER", "UNIT_DISPLAYPOWER"} },

	-- Max Mana in thousands
	["$km"] = { func = function(text, unit, self)
			local manamax = UnitPowerMax(unit)
			local txt = FormatValue(manamax, self.formatM, self.shortMana)
			text = DUF_gsub(text, '$km', txt, 1)
			return text
		end,
		events = {"UNIT_MAXPOWER", "UNIT_DISPLAYPOWER"} },

	-- Mana as a percent
	["$mp"] = { func = function(text, unit, self)
			local mana = UnitPower(unit)
			local manamax = UnitPowerMax(unit)
			local percent = 0
			if (manamax == 0) then
				percent = 0
			else
				percent = mathfloor(mana/manamax * 100)
			end
			text = DUF_gsub(text, '$mp', percent, 1)
			return text
		end,
		events = {"UNIT_MAXPOWER", "UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },

	-- Mana Used
	["$mx"] = { func = function(text, unit, self)
			local mana = UnitPower(unit)
			local manamax = UnitPowerMax(unit)
			local damage = manamax - mana
			text = DUF_gsub(text, '$mx', damage, 1)
			return text
		end,
		events = {"UNIT_MAXPOWER", "UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },

	-- Mana Used as a percent
	["$my"] = { func = function(text, unit, self)
			local mana = UnitPower(unit)
			local manamax = UnitPowerMax(unit)
			local damage = manamax - mana
			if (manamax == 0) then
				damage = 0
			else
				damage = mathfloor(damage/manamax * 100)
			end
			text = DUF_gsub(text, '$my', damage)
			return text
		end,
		events = {"UNIT_MAXPOWER", "UNIT_POWER_FREQUENT", "UNIT_DISPLAYPOWER"} },

	-- Race
	["$rc"] = { func = function(text, unit, self)
			local race = UnitRace(unit)
			if (not UnitName(unit)) then race = "race" end
			text = DUF_gsub(text, '$rc', race)
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },

	-- Class Abbreviation
	["$ca"] = { func = function(text, unit, self)
			local value = DUF_CLASSABBREV[UnitClass(unit)]
			if (not UnitName(unit)) then value = "CA" end
			text = DUF_gsub(text, '$ca', value)
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },

	-- Race Abbreviation
	["$ra"] = { func = function(text, unit, self)
			local value = DUF_RACEABBREV[UnitRace(unit)]
			if (not UnitName(unit)) then value = "RA" end
			text = DUF_gsub(text, '$ra', value)
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },

	-- Death Status - DEAD, GHOST, UNCONSCIOUS(for Target and TargetTarget) or nothing
	["$ds"] = { func = function(text, unit, self)
			local value
			if (UnitIsGhost(unit)) then
				value = DUF_TEXT.Ghost
			elseif (UnitIsDead(unit)) then
				value = DUF_TEXT.Dead
			elseif (not UnitName(unit)) then
				value = DUF_TEXT.Dead
			else
				value = nil
			end
			if (unit == "target" or unit == "targettarget") then
				if (UnitIsUnconscious(unit)) then
					value = DUF_TEXT.Unconscious
				end
			end
			text = DUF_gsub(text, '$ds', value)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT} },

	-- Faction - Horde, Alliance, or nothing
	["$fa"] = { func = function(text, unit, self)
			local _, value = UnitFactionGroup(unit)
			if (not UnitName(unit)) then value = "faction" end
			text = DUF_gsub(text, '$fa', value)
			return text
		end,
		events = {"UNIT_FACTION"} },

	-- Is Visible
	["$iv"] = { func = function(text, unit, self)
			local value
			if (not UnitIsVisible(unit)) then
				value = DUF_TEXT.OutOfRange
			end
			text = DUF_gsub(text, '$iv', value)
			return text
		end,
		events = {} },

	-- Item Level (player average)
	["$il"] = { func = function(text, unit, self)
			local value = ""
			if GetAverageItemLevel then
				value = select(2, GetAverageItemLevel())
			end
			if (not UnitIsVisible(unit)) then
				value = DUF_TEXT.OutOfRange
			end
			text = DUF_gsub(text, '$il', value)
			return text
		end,
		events = {} },

	-- In Combat - COMBAT or nothing
	["$ic"] = { func = function(text, unit, self)
			local value = ""
			if (UnitAffectingCombat(unit) or (not UnitName(unit))) then
				value = DUF_TEXT.Combat
			end
			text = DUF_gsub(text, '$ic', value)
			return text
		end,
		events = {} },

	-- Offline Status - OFFLINE or nothing
	["$of"] = { func = function(text, unit, self)
			local value
			if (UnitIsConnected(unit)) then
				value = nil
			else
				value = DUF_TEXT.Offline
			end
			text = DUF_gsub(text, '$of', value)
			return text
		end,
		events = {} },

	-- Creature Classification - Elite, Boss, etc.
	["$cc"] = { func = function(text, unit, self)
			local value = UnitClassification(unit)
			if (value == "normal") then
				value = nil
			elseif (value == "elite" or (not UnitName(unit))) then
				value = DUF_TEXT.Elite
			elseif (value == "worldboss") then
				value = DUF_TEXT.Boss
			elseif (value == "rare") then
				value = DUF_TEXT.Rare
			elseif (value == "rareelite") then
				value = DUF_TEXT.RareElite
			end
			text = DUF_gsub(text, '$cc', value)
			return text
		end,
		events = {"UNIT_CLASSIFICATION_CHANGED"} },

	-- Creature Type - Beast, Humanoid, Undead, etc.
	["$ct"] = { func = function(text, unit, self)
			local value = UnitCreatureType(unit)
			if (UnitIsPlayer(unit)) then value = nil end
			if (not UnitName(unit)) then value = "beast" end
			text = DUF_gsub(text, '$ct', value)
			return text
		end,
		events = {"UNIT_CLASSIFICATION_CHANGED"} },

	-- Creature Family - Bear, Crab, Cat, etc.
	["$cf"] = { func = function(text, unit, self)
			local value = UnitCreatureFamily(unit)
			if (not UnitName(unit)) then value = "cat" end
			text = DUF_gsub(text, '$cf', value)
			return text
		end,
		events = {"UNIT_CLASSIFICATION_CHANGED"} },

	-- Health Regen per Second
	["$hr"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$hr', self.healthregen)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT} },

	-- Mana Regen per Second
	["$mr"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$mr', self.manaregen)
			return text
		end,
		events = {"UNIT_POWER_FREQUENT"} },

	-- Health Regen per Tick
	["$ht"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$ht', self.healthregentick)
			return text
		end,
		events = {NS.UNIT_HEALTH_EVENT} },

	-- Mana Regen per Tick
	["$mt"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$mt', self.manaregentick)
			return text
		end,
		events = {"UNIT_POWER_FREQUENT"} },

	-- Recent Damage - shows all combat text: damage, blocks, evasions, etc.
	["$rd"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$rd', self.damagetext)
			return text
		end,
		events = {"UNIT_COMBAT"} },

	-- Recent Heals
	["$rh"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$rh', self.healtext)
			return text
		end,
		events = {"UNIT_COMBAT"} },

	-- Sex
	["$sx"] = { func = function(text, unit, self)
			local value = DUF_UnitSex(unit)
			text = DUF_gsub(text, '$sx', value)
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },

	-- Sex Abbreviation
	["$sa"] = { func = function(text, unit, self)
			local value = DUF_SEXABBREV[DUF_UnitSex(unit)]
			text = DUF_gsub(text, '$sa', value)
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },

	-- Tapped - TAPPED or nothing
	["$do"] = { func = function(text, unit, self)
			local value
--			if (not UnitPlayerControlled(unit) and UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) and not UnitIsTappedByAllThreatList(unit)) then
-- Legion			if (UnitIsTapped(unit) and (not UnitIsTappedByPlayer(unit))) then
			if not UnitPlayerControlled(unit) and UnitIsTapDenied(unit) then
				value = DUF_TEXT.Tapped
			end
			text = DUF_gsub(text, '$do', value)
			return text
		end,
		events = {"PLAYER_FLAGS_CHANGED"} },

	-- Unit Reaction - Hostile, Neutral, Friendly
	["$re"] = { func = function(text, unit, self)
			local value = UnitReaction("player", unit)
			if (value) then
				if (value < 4) then
					value = DUF_TEXT.Hostile
				elseif (value == 4) then
					value = DUF_TEXT.Neutral
				else
					value = DUF_TEXT.Friendly
				end
			end
			text = DUF_gsub(text, '$re', value)
			return text
		end,
		events = {} },

	--PVP Rank
	["$pk"] = { func = function(text, unit, self)
			local value = NS.GetPVPRankInfo(NS.UnitPVPRank(unit), unit)
			text = DUF_gsub(text, '$pk', value)
			return text
		end,
		events = {"UNIT_FACTION"} },

	--PVP Rank Number
	["$pn"] = { func = function(text, unit, self)
			local value = NS.UnitPVPRank(unit)
			if (value > 0) then
				value = value - 4
			else
				value = nil
			end
			text = DUF_gsub(text, '$pn', value)
			return text
		end,
		events = {"UNIT_FACTION"} },

	--PVP Tagged - PvP or PvP Free For All
	["$pt"] = { func = function(text, unit, self)
			local value
			if (UnitIsPVPFreeForAll(unit)) then
				value = DUF_TEXT.PVPFree
			elseif (UnitIsPVP(unit)) then
				value = DUF_TEXT.PVP
			end
			text = DUF_gsub(text, '$pt', value)
			return text
		end,
		events = {"UNIT_FACTION"} },

	--Combo Points as a number
	["$cp"] = { func = function(text, unit, self)
--			local value = GetComboPoints( UnitHasVehicleUI( "player") and "vehicle" or "player", "target")
			local value = UnitPower("player", Enum.PowerType.ComboPoints)
			if (value == 0) then
				value = ""
			end
			text = DUF_gsub(text, '$cp', value)
			return text
		end,
		events = {"UNIT_POWER_FREQUENT"} },

	--Mana Label - Mana, Energy, Rage, Focus, Runic Power
	["$ml"] = { func = function(text, unit, self)
			local value = UnitPowerType(unit)
			if (value == 0) then
				value = DUF_TEXT.Mana
			elseif (value == 1) then
				value = DUF_TEXT.Rage
			elseif (value == 2) then
				value = DUF_TEXT.Focus
			elseif (value == 3) then
				value = DUF_TEXT.Energy
			elseif (value == 6) then
				value = DUF_TEXT.RunicPower
			end
			text = DUF_gsub(text, '$ml', value)
			return text
		end,
		events = {"UNIT_DISPLAYPOWER"} },

	--Current XP
	["$xc"] = { func = function(text, unit, self)
			local currentxp = UnitXP(unit)
			if (not currentxp) then
				currentxp = 0
			end
			text = DUF_gsub(text, '$xc', currentxp)
			return text
		end,
		events = {"PLAYER_XP_UPDATE", "UPDATE_EXHAUSTION", "PLAYER_LEVEL_UP", "PLAYER_UPDATE_RESTING"} },

	--Total XP Needed to Level
	["$xl"] = { func = function(text, unit, self)
			local maxxp = UnitXPMax(unit)
			if (not maxxp) then
				maxxp = 0
			end
			text = DUF_gsub(text, '$xl', maxxp)
			return text
		end,
		events = {"PLAYER_XP_UPDATE", "UPDATE_EXHAUSTION", "PLAYER_LEVEL_UP", "PLAYER_UPDATE_RESTING"} },

	--Net XP Needed to Level
	["$xd"] = { func = function(text, unit, self)
			local currentxp = UnitXP(unit)
			if (not currentxp) then
				currentxp = 0
			end
			local maxxp = UnitXPMax(unit)
			if (not maxxp) then
				maxxp = 0
			end
			local value = maxxp - currentxp
			text = DUF_gsub(text, '$xd', value)
			return text
		end,
		events = {"PLAYER_XP_UPDATE", "UPDATE_EXHAUSTION", "PLAYER_LEVEL_UP", "PLAYER_UPDATE_RESTING"} },

	--Percent XP Gained This Level
	["$xy"] = { func = function(text, unit, self)
			local currentxp = UnitXP(unit)
			if (not currentxp) then
				currentxp = 0
			end
			local maxxp = UnitXPMax(unit)
			if (not maxxp) then
				maxxp = 0
			end
			local value
			if (maxxp == 0) then
				value = 0
			else
				value = mathfloor(currentxp / maxxp * 100)
			end
			text = DUF_gsub(text, '$xy', value)
			return text
		end,
		events = {"PLAYER_XP_UPDATE", "UPDATE_EXHAUSTION", "PLAYER_LEVEL_UP", "PLAYER_UPDATE_RESTING"} },

	--Percent XP Until Level
	["$xx"] = { func = function(text, unit, self)
			local currentxp = UnitXP(unit)
			if (not currentxp) then
				currentxp = 0
			end
			local maxxp = UnitXPMax(unit)
			if (not maxxp) then
				maxxp = 0
			end
			local value
			if (maxxp == 0) then
				value = 0
			else
				value = mathfloor(currentxp / maxxp * 100)
			end
			value = 100 - value
			text = DUF_gsub(text, '$xx', value)
			return text
		end,
		events = {"PLAYER_XP_UPDATE", "UPDATE_EXHAUSTION", "PLAYER_LEVEL_UP", "PLAYER_UPDATE_RESTING"} },

	--Rested XP
	["$xr"] = { func = function(text, unit, self)
			local value = GetXPExhaustion()
			if (value) then
				value = mathfloor(value / 2)
			else
				value = 0
			end
			text = DUF_gsub(text, '$xr', value)
			return text
		end,
		events = {"PLAYER_XP_UPDATE", "UPDATE_EXHAUSTION", "PLAYER_LEVEL_UP", "PLAYER_UPDATE_RESTING"} },
	
	 --Rested XP Percentage courtesy of Dsanai
	["$xb"] = { func = function(text, unit, self)
			local restXP = GetXPExhaustion()
			local nextlevelXP = UnitXPMax("player")
			local PercentRest = 0
			if (restXP) then
				PercentRest = DL_round(restXP / nextlevelXP * 100, 0)
			end
			text = DUF_gsub(text, '$xb', PercentRest)
			return text
		end,
		events = {"PLAYER_XP_UPDATE", "UPDATE_EXHAUSTION", "PLAYER_LEVEL_UP", "PLAYER_UPDATE_RESTING"} },

	--Creature Difficulty - Trivial, Minor, Suicide, etc.
	["$cd"] = { func = function(text, unit, self)
			local diff = UnitLevel(unit) - UnitLevel("player")
			local trivial = GetQuestGreenRange and GetQuestGreenRange() or UnitQuestTrivialLevelRange(unit)
			local difficulty
			if (UnitLevel(unit) == -1) then
				difficulty = 6
			elseif (diff < -trivial) then
				difficulty = 1
			elseif (diff < -2) then
				difficulty = 2
			elseif (diff < 3) then
				difficulty = 3
			elseif (diff < 5) then
				difficulty = 4
			elseif (diff < 11) then
				difficulty = 5
			else
				difficulty = 6
			end
			text = DUF_gsub(text, '$cd', DUF_TEXT["LevelDifference"..difficulty])
			return text
		end,
		events = {"UNIT_LEVEL"} },

	--Guild
	["$gu"] = { func = function(text, unit, self)
			local value = GetGuildInfo(unit)
			text = DUF_gsub(text, '$gu', value)
			return text
		end,
		events = {"UNIT_NAME_UPDATE"} },
	-- Unit Casting
	["$uc"] = { func = function(text, unit, self)
-- BFA		local cspell, crank, value, cicon, cstartTime, cendTime, cisTradeSkill = UnitCastingInfo(unit)
		local cspell, value, cicon, cstartTime, cendTime, cisTradeSkill = UnitCastingInfo(unit)
		if (cspell == nil) then
			cisChanneling = true
-- BFA			cspell, crank, value, cicon, cstartTime, cendTime, cisTradeSkill = UnitChannelInfo(unit)
			cspell, value, cicon, cstartTime, cendTime, cisTradeSkill = UnitChannelInfo(unit)
		end
			text = DUF_gsub(text, '$uc', value)
			return text
		end,
		events = {"UNIT_SPELLCAST_START", "UNIT_SPELLCAST_STOP", "UNIT_SPELLCAST_FAILED", "UNIT_SPELLCAST_INTERRUPTED", "UNIT_SPELLCAST_CHANNEL_START", "UNIT_SPELLCAST_CHANNEL_STOP"} },
	-- Unit Target's Name
	["$tn"] = { func = function(text, unit, self)
			local value = UnitName(unit.."target")
			if (not value) then
				value = DUF_TEXT.NoTarget
			elseif (UnitIsUnit(unit.."target", "player")) then
				value = DUF_TEXT.You
			elseif (UnitIsUnit(unit.."target", "target")) then
				value = DUF_TEXT.YourTarget
			end
			text = DUF_gsub(text, '$tn', value)
			return text
		end,
		events = {"UNIT_TARGET"} },

	-- Unit Target's Health
	["$th"] = { func = function(text, unit, self)
			local value = self.targethealth
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$th', value)
			return text
		end,
		events = {} },

	-- Unit Target's Max Health
	["$tx"] = { func = function(text, unit, self)
			local value = self.targethealthmax
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$tx', value)
			return text
		end,
		events = {} },

	-- Unit Target's Mana
	["$tm"] = { func = function(text, unit, self)
			local value = self.targetmana
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$tm', value)
			return text
		end,
		events = {} },

	-- Unit's Target's Max Mana
	["$ty"] = { func = function(text, unit, self)
			local value = self.targetmanamax
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$ty', value)
			return text
		end,
		events = {} },

	-- Unit's Target's Health Percent
	["$ta"] = { func = function(text, unit, self)
			local value
			if (self.targethealth and self.targethealthmax) then
				if (self.targethealthmax == 0) then
					value = 0
				else
					value = mathceil(self.targethealth / self.targethealthmax * 100)
				end
			end
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$ta', value)
			return text
		end,
		events = {} },
	-- Unit's Target's Mana Percent
	["$tb"] = { func = function(text, unit, self)
			local value
			if (self.targetmana and self.targetmanamax) then
				if (self.targetmanamax == 0) then
					value = 0
				else
					value = mathceil(self.targetmana / self.targetmanamax * 100)
				end
			end
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$tb', value)
			return text
		end,
		events = {} },

	-- Unit's Target's Level
	["$tl"] = { func = function(text, unit, self)
			local value = self.targetlevel
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$tl', value)
			return text
		end,
		events = {} },

	-- Unit's Target's Type
	["$tt"] = { func = function(text, unit, self)
			local value = self.targettype
			if (not self.targetname) then
				value = nil
			end
			text = DUF_gsub(text, '$tt', value)
			return text
		end,
		events = {} },

	-- Color Code
	["$co"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$co', '|c')
			return text
		end,
		events = {} },

	-- Default textbox colour
	["$cs"] = { func = function(text, unit, self)
			text = DUF_gsub(text, '$cs', '|r')
			return text
		end,
		events = {} },
	-- NPC
	["$np"] = { func = function(text, unit, self)
			local value
			if (not UnitIsPlayer(unit)) then
				value = "NPC"
			end
			text = DUF_gsub(text, '$np', value)
			return text
		end,
		events = {} },

	-- Shorthand Elite Text
	["$cx"] = { func = function(text, unit, self)
			local value = UnitClassification(unit)
			if (value == "normal") then
				value = nil
			elseif (value == "elite" or (not UnitName(unit))) then
				value = "+"
			elseif (value == "worldboss") then
				value = "++"
			elseif (value == "rare") then
				value = "(R)"
			elseif (value == "rareelite") then
				value = "(R)+"
			end
			text = DUF_gsub(text, '$cx', value)
			return text
		end,
		events = {"UNIT_CLASSIFICATION_CHANGED"} },

	-- Reaction Color Context
	["$cr"] = { func = function(text, unit, self)
			local value
			local r, g, b = NS:DUF_Get_ReactionColor(unit)
			if (r and g and b) then
				value = "|cFF"..strformat("%02X%02X%02X", r * 255.0, g * 255.0, b * 255.0)
			end
			text = DUF_gsub(text, '$cr', value)
			return text
		end,
		events = {} },

	-- Class Color Context
	["$cw"] = { func = function(text, unit, self)
			local value
			local r, g, b = NS:DUF_Get_ClassColor(unit)
			if (r and g and b) then
				value = "|cFF"..strformat("%02X%02X%02X", r * 255.0, g * 255.0, b * 255.0)
			end
			text = DUF_gsub(text, '$cw', value)
			return text
		end,
		events = {} },

	-- Mana Color Context
	["$cm"] = { func = function(text, unit, self)
			local value
			local r, g, b = NS:DUF_Get_ManaColor(unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].TextBox[self:GetID()].textcolor)
			if (r and g and b) then
				value = "|cFF"..strformat("%02X%02X%02X", r * 255.0, g * 255.0, b * 255.0)
			end
			text = DUF_gsub(text, '$cm', value)
			return text
		end,
		events = {} },

	-- Health Color Context
	["$ch"] = { func = function(text, unit, self)
			local value
			local r, g, b = NS:DUF_Get_HealthColor(unit, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].TextBox[self:GetID()].textcolor, UnitHealth(unit), UnitHealthMax(unit))
			if (r and g and b) then
				value = "|cFF"..strformat("%02X%02X%02X", r * 255.0, g * 255.0, b * 255.0)
			end
			text = DUF_gsub(text, '$ch', value)
			return text
		end,
		events = {} },

	-- Difficulty Color Context
	["$cq"] = { func = function(text, unit, self)
			local value
			local r, g, b = NS:DUF_Get_DifficultyColor(unit)
			if (r and g and b) then
				value = "|cFF"..strformat("%02X%02X%02X", r * 255.0, g * 255.0, b * 255.0)
			end
			text = DUF_gsub(text, '$cq', value)
			return text
		end,
		events = {} },

	-- Powertype Color
	["$cz"] = { func = function(text, unit, self, debug)
			local color, value
			local unit = self:GetParent().unit
			local pt = UnitPowerType(unit)
			color = POWERTABLE[pt]
			if not color then -- Needs a colour box for this power type added to the Mana bar options and powertypecolor added in NewSettings
				if debug then
					print("DUF_Variables.lua[local POWERTABLE: " .. DUF_TEXT["ManaColorNotFound"] .. " - " .. pt)
				end
				color = PowerBarColor[pt]
			else
				color = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[NS.Bar.Power][color]
			end
--[[
			if (pt == 0) then
				color = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[NS.Bar.Power].manacolor
			elseif (pt == 1) then
				color = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[NS.Bar.Power].ragecolor
			elseif (pt == 2) then
				color = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[NS.Bar.Power].focuscolor
			elseif (pt == 3) then
				color = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[NS.Bar.Power].energycolor
			elseif (pt == 6) then
				color = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[NS.Bar.Power].runicpowercolor
			elseif (pt == 13) then
				color = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[NS.Bar.Power].insanitycolor
			end
]]--
			if (color.r and color.g and color.b) then
				value = "|cFF"..strformat("%02X%02X%02X", color.r * 255.0, color.g * 255.0, color.b * 255.0)
			end
			text = DUF_gsub(text, '$cz', value)
			return text
		end,
		events = {"UNIT_DISPLAYPOWER"} },
	
	-- Watched Faction
	["$wf"] = { func = function(text, unit, self)
			local name = GetWatchedFactionInfo()
			if (not name) then name = "" end
			text = strgsub(text, '$wf', name)
			return text
		end,
		events = {"UPDATE_FACTION"} },
	
	-- Watched Faction Max
	["$wm"] = { func = function(text, unit, self)
			local _, _, min, max = GetWatchedFactionInfo()
			text = strgsub(text, '$wm', max - min)
			return text
		end,
		events = {"UPDATE_FACTION", "PLAYER_LEVEL_UP"} },
	
	-- Watched Faction Current
	["$wc"] = { func = function(text, unit, self)
			local _, _, min, _, value = GetWatchedFactionInfo()
			text = strgsub(text, '$wc', value - min)
			return text
		end,
		events = {"UPDATE_FACTION", "PLAYER_LEVEL_UP"} },
	
	-- Watched Faction Percent
	["$wp"] = { func = function(text, unit, self)
			local _, _, min, max, value = GetWatchedFactionInfo()
			local v = value-min
			local m = max - min
			if m == 0 then
				m = ""
			elseif v == 0 then
				m = 0
			else
				m = DL_round((v / m) *100, 0)
			end
			text = strgsub(text, '$wp', m)
			return text
		end,
		events = {"UPDATE_FACTION", "PLAYER_LEVEL_UP"} },

	-- Watched Faction Reaction
	["$wr"] = { func = function(text, unit, self)
			local _, reaction = GetWatchedFactionInfo()
			reaction = GetText("FACTION_STANDING_LABEL"..reaction, UnitSex("player"))
			if (not reaction) then reaction = "" end
			text = strgsub(text, '$wr', reaction)
			return text
		end,
		events = {"UPDATE_FACTION", "PLAYER_LEVEL_UP"} },

	-- Unit is out of phase
	["$pa"] = { func = function(text, unit, self)
--			local reason = NS.UnitPhaseReason or ""
			if not DUF_FRAME_DATA[unit] then
				return
			end
			local reason = NS.UnitPhaseReason(unit)
			local t = ""
			if reason then
				t = DUF_TEXT.Phased
			end
			text = DUF_gsub(text, '$pa', t)
			return text
		end,
		events = {"UNIT_PHASE", "PLAYER_FLAGS_CHANGED", "GROUP_ROSTER_UPDATE", "RAID_ROSTER_UPDATE", "PLAYER_ENTERING_WORLD"} },
	-- Out of phase reason
	["$pr"] = { func = function(text, unit, self)
			if not DUF_FRAME_DATA[unit] then
				return
			end
			local reason = NS.UnitPhaseReason(unit)
			local t = ""
			if reason and PartyUtil then -- PartyUtil Blizz. util function set
				t = PartyUtil.GetPhasedReasonString(reason, unit)
			end
--[[
			local reason = NS.UnitPhaseReason or ""
			local t = ""
			if reason == 0 then
				t = PARTY_PHASED_MESSAGE --DUF_TEXT.Phased
			elseif reason == 1 then
				t = PARTY_PLAYER_SHARDED_NEITHER_IN_PARTY_SHARD
			elseif reason == 2 then
				if C_PvP.IsWarModeDesired() then    
					t = ERR_PVP_TOGGLE_OFF
				else
					t = ERR_PVP_TOGGLE_ON
				end
			elseif reason == 3 then
				if UnitFactionGroup("player") == "Alliance" then
					t = PARTY_PLAYER_CHROMIE_TIME_SELF_ALLIANCE
				else
					t = PARTY_PLAYER_CHROMIE_TIME_SELF_HORDE
				end
			end
]]--
			text = DUF_gsub(text, '$pr', t)
			return text
		end,
		events = {"UNIT_PHASE", "PLAYER_FLAGS_CHANGED", "GROUP_ROSTER_UPDATE", "RAID_ROSTER_UPDATE", "PLAYER_ENTERING_WORLD"} },

-- Removed Variables
--[[
	-- Artifact Power Current
	["$ap"] = { func = function(text, unit, self)
				if unit ~= 'player' then return end
				if not HasArtifactEquipped() then
					text = DUF_gsub(text, '$ap', "")
					return text
				end
--				local itemID, altItemID, name, icon, totalXP, pointsSpent = C_ArtifactUI.GetEquippedArtifactInfo()
--				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP)
				local itemID, altItemID, name, icon, totalXP, pointsSpent, quality, artifactAppearanceID, appearanceModID, itemAppearanceID, altItemAppearanceID, altOnTop, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo()
				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP, artifactTier)
				local txt = FormatValue(artifactXP, self.formatH, self.shortHealth)
				text = DUF_gsub(text, '$ap', txt)
				return text
			end,
			events = {"PLAYER_ENTERING_WORLD", "ARTIFACT_XP_UPDATE", "UNIT_INVENTORY_CHANGED"} },
	-- Total Artifact Power for next level
	["$an"] = { func = function(text, unit, self)
				if unit ~= 'player' then return end
				if not HasArtifactEquipped() then
					text = DUF_gsub(text, '$an', "")
					return text
				end
--				local itemID, altItemID, name, icon, totalXP, pointsSpent = C_ArtifactUI.GetEquippedArtifactInfo()
--				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP)
				local itemID, altItemID, name, icon, totalXP, pointsSpent, quality, artifactAppearanceID, appearanceModID, itemAppearanceID, altItemAppearanceID, altOnTop, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo()
				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP, artifactTier)
				local txt = FormatValue(xpForNextPoint, self.formatH, self.shortHealth)
				text = DUF_gsub(text, '$an', txt)
				return text
			end,
			events = {"PLAYER_ENTERING_WORLD", "ARTIFACT_XP_UPDATE", "UNIT_INVENTORY_CHANGED"} },
	-- Artifact Power as a percent
	["$ac"] = { func = function(text, unit, self)
				if unit ~= 'player' then return end
				if not HasArtifactEquipped() then
					text = DUF_gsub(text, '$ac', "")
					return text
				end
--				local itemID, altItemID, name, icon, totalXP, pointsSpent = C_ArtifactUI.GetEquippedArtifactInfo()
--				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP)
				local itemID, altItemID, name, icon, totalXP, pointsSpent, quality, artifactAppearanceID, appearanceModID, itemAppearanceID, altItemAppearanceID, altOnTop, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo()
				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP, artifactTier)
--				local total = artifactXP + xpForNextPoint 
--				local total = artifactXP + xpForNextPoint 
				local percent = mathfloor((artifactXP/xpForNextPoint) * 100)
				text = DUF_gsub(text, '$ac', percent)
				return text
			end,
			events = {"PLAYER_ENTERING_WORLD", "ARTIFACT_XP_UPDATE", "UNIT_INVENTORY_CHANGED"} },
	--Artifact Power Percent to next Level
	["$al"] = { func = function(text, unit, self)
				if unit ~= 'player' then return end
				if not HasArtifactEquipped() then
					text = DUF_gsub(text, '$al', "")
					return text
				end
--				local itemID, altItemID, name, icon, totalXP, pointsSpent = C_ArtifactUI.GetEquippedArtifactInfo()
--				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP)
				local itemID, altItemID, name, icon, totalXP, pointsSpent, quality, artifactAppearanceID, appearanceModID, itemAppearanceID, altItemAppearanceID, altOnTop, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo()
				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP, artifactTier)
--				local total = artifactXP + xpForNextPoint
--				local value = (artifactXP/total) * 100
				local value = (artifactXP/xpForNextPoint) * 100
				local percent = mathfloor(100-value)
				text = DUF_gsub(text, '$al', percent)
				return text
			end,
			events = {"PLAYER_ENTERING_WORLD", "ARTIFACT_XP_UPDATE", "UNIT_INVENTORY_CHANGED"} },
	--Artifact Power total
	["$at"] = { func = function(text, unit, self)
				if unit ~= 'player' then return end
				if not HasArtifactEquipped() then
					text = DUF_gsub(text, '$at', "")
					return text
				end
				local itemID, altItemID, name, icon, totalXP, pointsSpent, quality, artifactAppearanceID, appearanceModID, itemAppearanceID, altItemAppearanceID, altOnTop, artifactTier = C_ArtifactUI.GetEquippedArtifactInfo()
				local numPoints, artifactXP, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP, artifactTier)

				local total = artifactXP + xpForNextPoint
				text = DUF_gsub(text, '$at', total)
				return text
			end,
			events = {"PLAYER_ENTERING_WORLD", "ARTIFACT_XP_UPDATE", "UNIT_INVENTORY_CHANGED"} },
	-- Keybinding
	["$kb"] = { func = function(text, unit, self)
			local value
			if (unit == "player") then
				value = DL_Get_KeybindingText("TARGETSELF")
			elseif (unit == "pet") then
				value = DL_Get_KeybindingText("TARGETPET")
			elseif (unit == "party1") then
				value = DL_Get_KeybindingText("TARGETPARTYMEMBER1")
			elseif (unit == "party2") then
				value = DL_Get_KeybindingText("TARGETPARTYMEMBER2")
			elseif (unit == "party3") then
				value = DL_Get_KeybindingText("TARGETPARTYMEMBER3")
			elseif (unit == "party4") then
				value = DL_Get_KeybindingText("TARGETPARTYMEMBER4")
			elseif (unit == "partypet1") then
				value = DL_Get_KeybindingText("TARGETPARTYPET1")
			elseif (unit == "partypet2") then
				value = DL_Get_KeybindingText("TARGETPARTYPET2")
			elseif (unit == "partypet3") then
				value = DL_Get_KeybindingText("TARGETPARTYPET3")
			elseif (unit == "partypet4") then
				value = DL_Get_KeybindingText("TARGETPARTYPET4")
			end
			text = DUF_gsub(text, '$kb', value)
			return text
		end,
		events = {"UPDATE_BINDINGS"} },
]]--
}
--if NS:IsValidVersion(WOW_PROJECT_CLASSIC, WOW_PROJECT_BURNING_CRUSADE_CLASSIC) then
if WOW_PROJECT_ID ~= WOW_PROJECT_MAINLINE then
	tinsert(NS.DUF_VARIABLE_FUNCTIONS["$ph"].events, "UNIT_HAPPINESS")
	tinsert(NS.DUF_VARIABLE_FUNCTIONS["$px"].events, "UNIT_PET_EXPERIENCE")
	tinsert(NS.DUF_VARIABLE_FUNCTIONS["$py"].events, "UNIT_PET_EXPERIENCE")
	tinsert(NS.DUF_VARIABLE_FUNCTIONS["$pc"].events, "UNIT_PET_EXPERIENCE")
	tinsert(NS.DUF_VARIABLE_FUNCTIONS["$pp"].events, "UNIT_PET_EXPERIENCE")
	tinsert(NS.DUF_VARIABLE_FUNCTIONS["$pg"].events, "UNIT_PET_EXPERIENCE")
end
